package frog;

import java.awt.*;

public class BetterFrog implements IFrog {
  
  /**
   * The current position of the frog.
   */
  private Point _pos = new Point(0,0);
  
  /**
   * Represents an abstract state of a frog.
   */
  private static abstract class AFrogState {
    /**
     * Accessor method for the color of the frog.
     * @return The color of the frog.
     */
    abstract Color getColor();
    
    /**
     * Move request to move the frog.
     * @return The new position of the frog.
     */
    abstract Point moveBy(Point delta);
    
    /**
     * Hits the frog.
     * @return The frog.
     */
    abstract IFrog getHit();
  }
  
  /**
   * Concrete instance of AFrogState representing the live state.
   */
  private AFrogState liveState = new AFrogState(){
    /**
     * Accessor method for the color of the frog.
     * @return Color.GREEN always.
     */
    public Color getColor() {
      return Color.GREEN;
    }
    
    /**
     * Move request to move the frog by translating the given amount.
     * @return The new position of the frog.
     */
    public Point moveBy(Point delta) {
      _pos.translate(delta.x, delta.y);
      return _pos;
    }
    /**
     * Hits the frog.  The frog will enter the dead state.
     * @return The frog.
     */
    public IFrog getHit() {
      _state = deadState;
      return BetterFrog.this;
    }
  };
  
  /**
   * Concrete instance of AFrogState representing the dead state.
   */
  private AFrogState deadState = new AFrogState(){
    /**
     * Accessor method for the color of the frog.
     * @return Color.RED always.
     */
    public Color getColor() {
      return Color.RED;
    }
    
    /**
     * Move request to move the frog--No effect.
     * @return The current position of the frog.
     */
    public Point moveBy(Point delta) {
      return _pos;
    }
    /**
     * Hits the frog.   No effect.
     * @param context The context of this state.
     * @return The frog.
     */
    public IFrog getHit() {
      return BetterFrog.this;
    }
  };
  
  /**
   * The current state of the frog.
   */
  private AFrogState _state = liveState;
  
  /**
   * Accessor method for the position of the frog.
   * @return The current position Point of the frog
   */
  public Point getPos() {
    return _pos;
  }
  
  /**
   * Accessor method for the color of the frog.
   * A live frog is green and a dead frog is red.
   * @return The Color of the frog
   */
  public Color getColor() {
    return _state.getColor();
  }
  
  /**
   * Requests that the frog be translated the position by the given vector (Point).   
   * Live frogs move by the given amount.   Dead frogs do not move.
   * @param delta The amount to move the frog by.
   * @return The resultant position of the frog
   */
  public Point moveBy(Point delta) {
    return _state.moveBy(delta);
  }
  
  /**
   * The hits the frog.   
   * A live frog will die when it gets hit.   
   * A dead frog stays dead when gets hit. 
   * @return The frog itself is returned.
   */
  public IFrog getHit() {
    return _state.getHit();
  }  
}
